@@ -1,6 +1,6 @@
 
 
-### Copyright 2001-2003 Deepayan Sarkar <deepayan@stat.wisc.edu>
+### Copyright (C) 2001-2005 Deepayan Sarkar <Deepayan.Sarkar@R-project.org>
 ###
 ### This file is part of the lattice library for R.
 ### It is made available under the terms of the GNU General Public
@@ -25,9 +25,8 @@
 
 prepanel.default.bwplot <-
     function(x, y, box.ratio,
-             horizontal = TRUE,
+             horizontal = TRUE, nlevels,
              origin = NULL, stack = FALSE,
-             ## levels.fos = if (horizontal) unique(y) else unique(x), 
              ...)
 {
     ## This function needs to work for all high level functions in the
@@ -36,13 +35,19 @@ prepanel.default.bwplot <-
     ## getting the ranges. For stacked barcharts, things are slightly
     ## complicated.
 
-    if (length(x) && length(y)) {
-        
+    if (length(x) && length(y))
+    {
         #if (!is.numeric(x)) x <- as.numeric(x)
         #if (!is.numeric(y)) y <- as.numeric(y)
 
         temp <- .5  #* box.ratio/(box.ratio+1)
         if (horizontal)
+        {
+            if (!is.factor(y)) ## y came from a shingle
+            {
+                if (missing(nlevels)) nlevels <- length(unique(y))
+                y <- factor(y, levels = 1:nlevels)
+            }
             list(xlim =
                  if (stack) {
                      foo1 <- if (any(x > 0)) range( by(x[x>0], y[x>0, drop = TRUE], sum)) else 0
@@ -51,17 +56,20 @@ prepanel.default.bwplot <-
                  }
                  else if (is.numeric(x)) range(x[is.finite(x)], origin)
                  else levels(x),
-                 ylim =
-                 if (is.numeric(y)) range(y[is.finite(y)])
-                 else levels(y),
-                 ##ylim = c(1-temp, levels.fos + temp),
-                 yat = if (is.factor(y)) sort(unique(as.numeric(y))) else NULL,
+                 ylim = levels(y),
+                 yat = sort(unique(as.numeric(y))),
                  dx = 1,
                  dy = 1)
-        else 
-            list(xlim = if (is.numeric(x)) range(x[is.finite(x)]) else levels(x),
-                 ##xlim = c(1-temp, levels.fos + temp),
-                 xat = if (is.factor(x)) sort(unique(as.numeric(x))) else NULL,
+        }
+        else
+        {
+            if (!is.factor(x)) ## x came from a shingle
+            {
+                if (missing(nlevels)) nlevels <- length(unique(x))
+                x <- factor(x, levels = 1:nlevels)
+            }
+            list(xlim = levels(x),
+                 xat = sort(unique(as.numeric(x))),
                  ylim =
                  if (stack) {
                      foo1 <- if (any(y > 0)) range( by(y[y>0], x[y>0], sum)) else 0
@@ -72,6 +80,7 @@ prepanel.default.bwplot <-
                  else levels(y),
                  dx = 1,
                  dy = 1)
+        }
     }
     else list(xlim = c(NA, NA),
               ylim = c(NA, NA),
@@ -112,12 +121,15 @@ panel.barchart <-
     groupSub <- function(groups, subscripts, ...)
         groups[subscripts]
 
-    if (horizontal) {
+    if (horizontal)
+    {
 
         ## No grouping
 
-        if (is.null(groups)) {
-            if (is.null(origin)) {
+        if (is.null(groups))
+        {
+            if (is.null(origin))
+            {
                 origin <- current.viewport()$xscale[1]
                 reference <- FALSE
             }
@@ -141,7 +153,8 @@ panel.barchart <-
 
         ## grouped, with stacked bars
 
-        else if (stack) {
+        else if (stack)
+        {
 
             if (!is.null(origin) && origin != 0)
                 warning("origin forced to 0 for stacked bars")
@@ -150,7 +163,11 @@ panel.barchart <-
             vals <- sort(unique(groups))
             nvals <- length(vals)
             groups <- groupSub(groups, ...)
+
             col <- rep(col, length = nvals)
+            border <- rep(border, length = nvals)
+            lty <- rep(lty, length = nvals)
+            lwd <- rep(lwd, length = nvals)
 
             height <- box.ratio/(1 + box.ratio)
 
@@ -160,7 +177,8 @@ panel.barchart <-
                              lty = reference.line$lty,
                              lwd = reference.line$lwd)
 
-            for (i in unique(y)) {
+            for (i in unique(y))
+            {
                 ok <- y == i
                 ord <- sort.list(groups[ok])
                 pos <- x[ok][ord] > 0
@@ -196,8 +214,10 @@ panel.barchart <-
 
         ## grouped, with side by side bars
 
-        else {
-            if (is.null(origin)) {
+        else
+        {
+            if (is.null(origin))
+            {
                 origin <- current.viewport()$xscale[1]
                 reference <- FALSE
             }
@@ -205,7 +225,11 @@ panel.barchart <-
             vals <- sort(unique(groups))
             nvals <- length(vals)
             groups <- groupSub(groups, ...)
+
             col <- rep(col, length = nvals)
+            border <- rep(border, length = nvals)
+            lty <- rep(lty, length = nvals)
+            lwd <- rep(lwd, length = nvals)
 
             height <- box.ratio/(1 + nvals * box.ratio)
             if (reference)
@@ -213,7 +237,8 @@ panel.barchart <-
                              col = reference.line$col,
                              lty = reference.line$lty,
                              lwd = reference.line$lwd)
-            for (i in unique(y)) {
+            for (i in unique(y))
+            {
                 ok <- y == i
                 nok <- sum(ok)
                 grid.rect(gp =
@@ -233,9 +258,12 @@ panel.barchart <-
     
     ## if not horizontal
 
-    else {
-        if (is.null(groups)) {
-            if (is.null(origin)) {
+    else
+    {
+        if (is.null(groups))
+        {
+            if (is.null(origin))
+            {
                 origin <- current.viewport()$yscale[1]
                 reference <- FALSE
             }
@@ -257,7 +285,8 @@ panel.barchart <-
                       just = c("centre", "bottom"),
                       default.units = "native")
         }
-        else if (stack) {
+        else if (stack)
+        {
 
             if (!is.null(origin) && origin != 0)
                 warning("origin forced to 0 for stacked bars")
@@ -266,7 +295,11 @@ panel.barchart <-
             vals <- sort(unique(groups))
             nvals <- length(vals)
             groups <- groupSub(groups, ...)
+
             col <- rep(col, length = nvals)
+            border <- rep(border, length = nvals)
+            lty <- rep(lty, length = nvals)
+            lwd <- rep(lwd, length = nvals)
 
             width <- box.ratio/(1 + box.ratio)
 
@@ -276,7 +309,8 @@ panel.barchart <-
                              lty = reference.line$lty,
                              lwd = reference.line$lwd)
 
-            for (i in unique(x)) {
+            for (i in unique(x))
+            {
                 ok <- x == i
                 ord <- sort.list(groups[ok])
                 pos <- y[ok][ord] > 0
@@ -311,8 +345,10 @@ panel.barchart <-
 
             
         }
-        else {
-            if (is.null(origin)) {
+        else
+        {
+            if (is.null(origin))
+            {
                 origin <- current.viewport()$yscale[1]
                 reference = FALSE
             }
@@ -320,7 +356,11 @@ panel.barchart <-
             vals <- sort(unique(groups))
             nvals <- length(vals)
             groups <- groupSub(groups, ...)
+
             col <- rep(col, length = nvals)
+            border <- rep(border, length = nvals)
+            lty <- rep(lty, length = nvals)
+            lwd <- rep(lwd, length = nvals)
 
             width <- box.ratio/(1 + nvals * box.ratio)
             if (reference)
@@ -328,7 +368,8 @@ panel.barchart <-
                              col = reference.line$col,
                              lty = reference.line$lty,
                              lwd = reference.line$lwd)
-            for (i in unique(x)) {
+            for (i in unique(x))
+            {
                 ok <- x == i
                 nok <- sum(ok)
                 grid.rect(gp =
@@ -370,8 +411,6 @@ panel.dotplot <-
     if (horizontal)
     {
         yscale <- current.viewport()$yscale
-        ##if (is.null(levels.fos))
-        ##    levels.fos <- floor(yscale[2])-ceiling(yscale[1])+1
         panel.abline(h = levels.fos,
                      col = col.line,
                      lty = lty, lwd = lwd)
@@ -388,8 +427,6 @@ panel.dotplot <-
     else
     {
         xscale <- current.viewport()$xscale
-        ##if (is.null(levels.fos))
-        ##    levels.fos <- floor(xscale[2])-ceiling(xscale[1])+1
         panel.abline(v = levels.fos, col = col.line,
                      lty = lty, lwd = lwd)
         if (is.null(groups)) 
@@ -431,14 +468,18 @@ panel.stripplot <-
 
 
 panel.bwplot <-
-    function(x, y, box.ratio=1, horizontal = TRUE, pch=box.dot$pch,
-             col = box.dot$col, cex = box.dot$cex,
+    function(x, y, box.ratio = 1,
+             horizontal = TRUE,
+             pch = box.dot$pch,
+             col = box.dot$col,
+             cex = box.dot$cex,
              font = box.dot$font,
              fontfamily = box.dot$fontfamily,
              fontface = box.dot$fontface, 
-             fill = box.rectangle$fill, varwidth = FALSE,
+             fill = box.rectangle$fill,
+             varwidth = FALSE,
              levels.fos = if (horizontal) unique(y) else unique(x),
-             coef = 1.5, ...)
+             coef = 1.5, do.out = TRUE, ...)
 {
     x <- as.numeric(x)
     y <- as.numeric(y)
@@ -451,27 +492,21 @@ panel.bwplot <-
     plot.symbol <- trellis.par.get("plot.symbol")
 
     fontsize.points <- trellis.par.get("fontsize")$points
-
-    ## In case levels.fos is not given (which should not happen), I'll
-    ## be working on the premise that EACH INTEGER in the y-RANGE is a
-    ## potential location of a boxplot. 
+    xscale <- current.viewport()$xscale
+    yscale <- current.viewport()$yscale
 
     if (horizontal)
     {
 
         maxn <- max(by(x, y, length)) ## used if varwidth = TRUE
-
-        yscale <- current.viewport()$yscale
-        ##if (is.null(levels.fos)) levels.fos <- floor(yscale[2])-ceiling(yscale[1])+1
-        lower <- ceiling(yscale[1])
+        ##lower <- ceiling(yscale[1])
         height <- box.ratio/(1+box.ratio)
-        xscale <- current.viewport()$xscale
-        ##if (levels.fos > 0)
+
         for (yval in levels.fos)
         {
 
             ## yval  <- i
-            stats <- boxplot.stats(x[y==yval], coef = coef)
+            stats <- boxplot.stats(x[y==yval], coef = coef, do.out = do.out)
             
             if (stats$n>0)
             {
@@ -539,17 +574,13 @@ panel.bwplot <-
     {
 
         maxn <- max(by(y, x, length)) ## used if varwidth = TRUE
-
-        xscale <- current.viewport()$xscale
-        ## if (is.null(levels.fos)) levels.fos <- floor(xscale[2])-ceiling(xscale[1])+1
-        lower <- ceiling(xscale[1])
+        ##lower <- ceiling(xscale[1])
         width <- box.ratio/(1+box.ratio)
-        yscale <- current.viewport()$yscale
-        ##if (levels.fos > 0)
-        ##for (i in 1:levels.fos) {
-        for (xval in levels.fos) {
+
+        for (xval in levels.fos)
+        {
             ##xval  <- i
-            stats <- boxplot.stats(y[x==xval], coef = coef)
+            stats <- boxplot.stats(y[x==xval], coef = coef, do.out = do.out)
 
             if (stats$n>0)
             {
@@ -615,6 +646,141 @@ panel.bwplot <-
     }
 }
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+panel.violin <-
+    function(x, y, box.ratio = 1, horizontal = TRUE,
+
+             alpha = bar.fill$alpha,
+             border = bar.fill$border,
+             lty = bar.fill$lty,
+             lwd = bar.fill$lwd,
+             col = bar.fill$col,
+
+             varwidth = FALSE,
+
+             bw = NULL,
+             adjust = NULL,
+             kernel = NULL,
+             window = NULL,
+             width = NULL,
+             n = 50,
+             from = NULL,
+             to = NULL,
+             cut = NULL,
+             na.rm = TRUE,
+             
+             ...)
+{
+    x <- as.numeric(x)
+    y <- as.numeric(y)
+
+    if (length(x) < 1) return()
+
+    ##reference.line <- trellis.par.get("reference.line")
+    bar.fill <- trellis.par.get("bar.fill")
+
+    ## density doesn't handle unrecognized arguments (not even to
+    ## ignore it).  A tedious but effective way to handle that is to
+    ## have all arguments to density be formal arguments to this panel
+    ## function, as follows:
+
+    darg <- list()
+    darg$bw <- bw
+    darg$adjust <- adjust
+    darg$kernel <- kernel
+    darg$window <- window
+    darg$width <- width
+    darg$n <- n
+    darg$from <- from
+    darg$to <- to
+    darg$cut <- cut
+    darg$na.rm <- na.rm
+
+    my.density <- function(x) do.call("density", c(list(x = x), darg))
+
+    numeric.list <- if (horizontal) split(x, factor(y)) else split(y, factor(x))
+    levels.fos <- as.numeric(names(numeric.list))
+    d.list <- lapply(numeric.list, my.density)
+    ## n.list <- sapply(numeric.list, length)  UNNECESSARY
+    dx.list <- lapply(d.list, "[[", "x")
+    dy.list <- lapply(d.list, "[[", "y")
+
+    max.d <- sapply(dy.list, max)
+    if (varwidth) max.d[] <- max(max.d)
+
+    ##str(max.d)
+    
+    xscale <- current.viewport()$xscale
+    yscale <- current.viewport()$yscale
+    height <- box.ratio / (1 + box.ratio)
+
+    if (horizontal)
+    {
+        for (i in seq(along = levels.fos))
+        {
+            pushViewport(viewport(y = unit(levels.fos[i], "native"),
+                                  height = unit(height, "native"),
+                                  yscale = c(max.d[i] * c(-1, 1)),
+                                  xscale = xscale))
+            grid.polygon(x = c(dx.list[[i]], rev(dx.list[[i]])),
+                         y = c(dy.list[[i]], -rev(dy.list[[i]])),
+                         default.units = "native",
+                         gp = gpar(fill = col, col = border, lty = lty, lwd = lwd, alpha = alpha))
+            popViewport()
+        }
+    }
+    else
+    {
+        for (i in seq(along = levels.fos))
+        {
+            pushViewport(viewport(x = unit(levels.fos[i], "native"),
+                                  width = unit(height, "native"),
+                                  xscale = c(max.d[i] * c(-1, 1)),
+                                  yscale = yscale))
+            grid.polygon(y = c(dx.list[[i]], rev(dx.list[[i]])),
+                         x = c(dy.list[[i]], -rev(dy.list[[i]])),
+                         default.units = "native",
+                         gp = gpar(fill = col, col = border, lty = lty, lwd = lwd, alpha = alpha))
+            popViewport()
+        }
+    }
+    invisible()
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -806,8 +972,12 @@ bwplot <-
     x <- form$right
     y <- form$left
     if (is.null(y))
+    {
         y <- rep(if (is.null(names(x))) '' else names(x), length = length(x))
-    if (number.of.cond == 0) {
+        y <- factor(y, levels = unique(y))
+    }
+    if (number.of.cond == 0)
+    {
         strip <- FALSE
         cond <- list(as.factor(rep(1, length(x))))
         number.of.cond <- 1
@@ -945,11 +1115,19 @@ bwplot <-
     foo$panel.args.common <- dots
     foo$panel.args.common$box.ratio <- box.ratio
     foo$panel.args.common$horizontal <- horizontal
-##    foo$panel.args.common$levels.fos <- ## fos - the factor/shingle in x/y
-##        if (horizontal) num.l.y else num.l.x
     if (subscripts) foo$panel.args.common$groups <- groups
 
-
+    ## only used if shingle, important if some levels are missing
+    if (horizontal)
+    {
+        if (!is.f.y) ## y shingle
+            foo$panel.args.common$nlevels <- num.l.y
+    }
+    else
+    {
+        if (!is.f.x) ## x shingle
+            foo$panel.args.common$nlevels <- num.l.x
+    }
 
     nplots <- prod(cond.max.level)
     if (nplots != prod(sapply(foo$condlevels, length))) stop("mismatch")
@@ -1070,8 +1248,11 @@ bwplot <-
         foo$legend <-
             list(list(fun = "drawSimpleKey",
                       args =
-                      c(list(levels(as.factor(groups))),
-                        if (is.list(auto.key)) auto.key else list())))
+                      updateList(list(text = levels(as.factor(groups)),
+                                      points = TRUE,
+                                      rectangles = FALSE,
+                                      lines = FALSE), 
+                                 if (is.list(auto.key)) auto.key else list())))
         foo$legend[[1]]$x <- foo$legend[[1]]$args$x
         foo$legend[[1]]$y <- foo$legend[[1]]$args$y
         foo$legend[[1]]$corner <- foo$legend[[1]]$args$corner
