@@ -23,7 +23,14 @@
 #include <stdio.h>
 #include <Rversion.h>
 
-extern char *getRHOME(); /* in ../rhome.c */
+extern char *getRHOME(), *getRUser(); /* in ../rhome.c */
+
+void R_Suicide(char *s) /* for use in ../rhome.o */
+{
+    fprintf(stderr, "FATAL ERROR:%s\n", s);
+    exit(2);
+}
+
 
 static int pwait(HANDLE p)
 {
@@ -64,11 +71,12 @@ int rcmdfn (int cmdarg, int argc, char **argv)
        set PATH to include R_HOME\bin
        set PERL5LIB to %R_HOME%/share/perl;%Perl5LIB%
        set TEXINPUTS to %R_HOME%/share/texmf;%TEXINPUTS%
+       set HOME if unset
        launch %R_HOME%\bin\$*
      */
     int i, iused, res, status = 0;
     char *RHome, PERL5LIB[MAX_PATH], TEXINPUTS[MAX_PATH], PATH[10000],
-	RHOME[MAX_PATH], *p, cmd[10000], Rversion[25];
+	RHOME[MAX_PATH], *p, cmd[10000], Rversion[25], HOME[MAX_PATH + 10];
     char RCMD[] = "R CMD";
     int len = strlen(argv[0]);
     
@@ -105,7 +113,7 @@ int rcmdfn (int cmdarg, int argc, char **argv)
 
     if (cmdarg > 0 && argc > cmdarg && strcmp(argv[cmdarg], "BATCH") == 0) {
 	/* handle Rcmd BATCH internally */
-	char infile[MAX_PATH], outfile[MAX_PATH], cmd[MAX_PATH];
+	char infile[MAX_PATH], outfile[MAX_PATH];
 	DWORD ret;
 	SECURITY_ATTRIBUTES sa;
 	PROCESS_INFORMATION pi;
@@ -146,6 +154,10 @@ int rcmdfn (int cmdarg, int argc, char **argv)
 		break;
 	    }
 	    if (argv[i][0] == '-') {
+		if (strlen(cmd) + strlen(argv[i]) > 9900) {
+		    fprintf(stderr, "command line too long\n");
+		    return(27);
+		}
 		strcat(cmd, " ");
 		strcat(cmd, argv[i]);
 		iused = i;
@@ -206,8 +218,7 @@ int rcmdfn (int cmdarg, int argc, char **argv)
 	return(pwait(pi.hProcess));
     } else {
 	RHome = getRHOME();
-	if (argc > cmdarg+1 && strcmp(argv[cmdarg+1], "RHOME") == 0) {
-	    fprintf(stdout, "%s", RHome);
+	if (argc > cmdarg+1 && strcmp(argv[cmdarg+1], "RHOME") == 0) {	    fprintf(stdout, "%s", RHome);
 	    return(0);
 	}
 	strcpy(RHOME, "R_HOME=");
@@ -242,6 +253,11 @@ int rcmdfn (int cmdarg, int argc, char **argv)
 	if ( (p = getenv("TEXINPUTS")) ) strcat(TEXINPUTS, p);
 	putenv(TEXINPUTS);
 
+	if( !getenv("HOME") ) {
+	    strcpy(HOME, "HOME=");
+	    strcat(HOME, getRUser());
+	    putenv(HOME);
+	}
 	if (cmdarg > 0 && argc > cmdarg) {
 	    p = argv[cmdarg];
 	    if (strcmp(p, "Rd2dvi") == 0) {
