@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000-2004   The R Development Core Team.
+ *  Copyright (C) 2000-2005   The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@ struct Rconn {
     void (*destroy)(struct Rconn *); /* when closing connection */
     int (*vfprintf)(struct Rconn *, const char *, va_list);
     int (*fgetc)(struct Rconn *);
+    int (*fgetc_internal)(struct Rconn *);
 /*    int (*ungetc)(int c, struct Rconn *); */
     double (*seek)(struct Rconn *, double, int, int);
     void (*truncate)(struct Rconn *);
@@ -47,7 +48,14 @@ struct Rconn {
     int nPushBack, posPushBack; /* number of lines, position on top line */
     char **PushBack;
     int save, save2;
-    unsigned char encoding[256];
+    /* unsigned char encoding[256];*/
+    char encname[101];
+    /* will be iconv_t, which is a pointer. NULL if not in use */
+    void *inconv, *outconv;
+    /* The idea here is that no MBCS char will ever not fit */
+    char iconvbuff[25], oconvbuff[50], *next, init_out[25];
+    short navail, inavail;
+    Rboolean EOF_signalled;
     void *private;
 };
 
@@ -55,8 +63,12 @@ typedef struct fileconn {
     FILE *fp;
 #if defined(HAVE_OFF_T) && defined(__USE_LARGEFILE)
     off_t rpos, wpos;
+#else
+#ifdef Win32
+    off64_t rpos, wpos;
 #else
     long rpos, wpos;
+#endif
 #endif
     Rboolean last_was_write;
 } *Rfileconn;
@@ -108,13 +120,11 @@ typedef struct bzfileconn {
     void *bfp;
 } *Rbzfileconn;
 
-#ifdef Win32
 typedef struct clpconn {
     char *buff;
     int pos, len, last, sizeKB;
     Rboolean warned;
 } *Rclpconn;
-#endif
 
 /* zlib wants to use ZLIB_H without leading underscore in 1.2.1 */
 #if defined(_ZLIB_H) || defined(ZLIB_H)
@@ -145,11 +155,15 @@ Rconnection R_newurl(char *description, char *mode);
 Rconnection R_newsock(char *host, int port, int server, char *mode);
 Rconnection in_R_newsock(char *host, int port, int server, char *mode);
 Rconnection R_newunz(char *description, char *mode);
+int dummy_fgetc(Rconnection con);
 int dummy_vfprintf(Rconnection con, const char *format, va_list ap);
 int getActiveSink(int n);
+void con_pushback(Rconnection con, Rboolean newLine, char *line);
 
 int Rsockselect(int nsock, int *insockfd, int *ready, int *write,
 		double timeout);
 
+#define set_iconv Rf_set_iconv
+void set_iconv(Rconnection con);
 #endif
 
