@@ -9,7 +9,7 @@ as.dendrogram.hclust <- function (object, hang = -1, ...)
     z <- list()
     nMerge <- length(oHgt <- object$height)
     if (nMerge != nrow(object$merge))
-	stop("`merge' and `height' do not fit!")
+	stop("'merge' and 'height' do not fit!")
     hMax <- oHgt[nMerge]
     one <- 1:1;	   two <- 2:2 # integer!
     for (k in 1:nMerge) {
@@ -60,7 +60,7 @@ as.dendrogram.hclust <- function (object, hang = -1, ...)
     z
 }
 
-### MM: `FIXME'	 (2002-05-14):
+### MM: 'FIXME'	 (2002-05-14):
 ###	 =====
 ## We currently (mis)use a node's "members" attribute for two things:
 ## 1) #{sub nodes}
@@ -114,7 +114,7 @@ midcache.dendrogram <- function (x, type = "hclust")
 ##  Martin Maechler, 15 May 2002
 print.dendrogram <- function(x, digits = getOption("digits"), ...)
 {
-    cat("`dendrogram' ")
+    cat("'dendrogram' ")
     if(is.leaf(x))
 	cat("leaf '", format(attr(x, "label"), digits = digits),"'", sep='')
     else
@@ -134,7 +134,7 @@ function (object, max.level = 0, digits.d = 3, give.attr = FALSE,
 ## -- e.g. when 'object' is part of a larger structure which *is* str()ed
 ## with default max.level= 0,  the dendrogram shouldn't be str()ed to all levels
 
-    ## FIXME: `wid' argument is currently disregarded
+    ## FIXME: 'wid' argument is currently disregarded
     pasteLis <- function(lis, dropNam, sep = " = ") {
 	## drop uninteresting "attributes" here
 	lis <- lis[!(names(lis) %in% dropNam)]
@@ -142,9 +142,7 @@ function (object, max.level = 0, digits.d = 3, give.attr = FALSE,
 	paste(paste(names(fl), fl, sep=sep), collapse = ", ")
     }
 
-    nind <- nchar(istr <- indent.str)
-    if(substr(istr, nind,nind) == " ")
-       substr(istr, nind,nind) <- "`"
+    istr <- sub(" $", "`", indent.str)
     cat(istr, stem, sep="")
 
     at <- attributes(object)
@@ -420,7 +418,7 @@ cut.dendrogram <- function(x, h, ...)
     assignNodes <- function(subtree, h) {
 	if(!is.leaf(subtree)) {
 	    if(!(K <- length(subtree)))
-		stop("non leaf ", sQuote("subtree"), " of length 0 !!")
+		stop("non-leaf subtree of length 0")
 	    new.mem <- 0:0
 	    for(k in 1:K) {
                 sub <- subtree[[k]]
@@ -458,7 +456,7 @@ is.leaf <- function(object) (is.logical(L <- attr(object, "leaf"))) && L
 ## *Not* a method (yet):
 order.dendrogram <- function(x) {
     if( !inherits(x, "dendrogram") )
-	stop("order.dendrogram requires a dendrogram")
+	stop("'order.dendrogram' requires a dendrogram")
     unlist(x)
 }
 
@@ -557,13 +555,13 @@ function (x, Rowv=NULL, Colv=if(symm)"Rowv" else NULL,
 {
     scale <- if(symm && missing(scale)) "none" else match.arg(scale)
     if(length(di <- dim(x)) != 2 || !is.numeric(x))
-	stop("`x' must be a numeric matrix")
+	stop("'x' must be a numeric matrix")
     nr <- di[1]
     nc <- di[2]
     if(nr <= 1 || nc <= 1)
-	stop("`x' must have at least 2 rows and 2 columns")
+	stop("'x' must have at least 2 rows and 2 columns")
     if(!is.numeric(margins) || length(margins) != 2)
-	stop("`margins' must be a numeric vector of length 2")
+	stop("'margins' must be a numeric vector of length 2")
 
     doRdend <- !identical(Rowv,NA)
     doCdend <- !identical(Colv,NA)
