@@ -8,11 +8,11 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
     alternative <- match.arg(alternative)
 
     if(!missing(mu) && (length(mu) != 1 || is.na(mu)))
-        stop("mu must be a single number")
+        stop("'mu' must be a single number")
     if(!missing(conf.level) &&
        (length(conf.level) != 1 || !is.finite(conf.level) ||
         conf.level < 0 || conf.level > 1))
-        stop("conf.level must be a single number between 0 and 1")
+        stop("'conf.level' must be a single number between 0 and 1")
     if( !is.null(y) ) {
 	dname <- paste(deparse(substitute(x)),"and",
 		       deparse(substitute(y)))
@@ -26,7 +26,7 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
     }
     else {
 	dname <- deparse(substitute(x))
-	if( paired ) stop("y is missing for paired test")
+	if( paired ) stop("'y' is missing for paired test")
 	xok <- !is.na(x)
 	yok <- NULL
     }
@@ -36,7 +36,7 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
 	y <- NULL
     }
     nx <- length(x)
-    if(nx < 2) stop("not enough x observations")
+    if(nx < 2) stop("not enough 'x' observations")
     mx <- mean(x)
     vx <- var(x)
     estimate <- mx
@@ -50,7 +50,7 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
 	names(estimate) <- ifelse(paired,"mean of the differences","mean of x")
     } else {
 	ny <- length(y)
-	if(ny < 2) stop("not enough y observations")
+	if(ny < 2) stop("not enough 'y' observations")
 	my <- mean(y)
 	vy <- var(y)
 	method <- paste(if(!var.equal)"Welch", "Two Sample t-test")
@@ -104,7 +104,7 @@ function(formula, data, subset, na.action, ...)
        || (length(formula) != 3)
        || (length(attr(terms(formula[-2]), "term.labels")) != 1)
        || (length(attr(terms(formula[-3]), "term.labels")) != 1))
-        stop("formula missing or incorrect")
+        stop("'formula' missing or incorrect")
     m <- match.call(expand.dots = FALSE)
     if(is.matrix(eval(m$data, parent.frame())))
         m$data <- as.data.frame(data)
