@@ -810,7 +810,26 @@ void tcltk_init(void)
 
 }
 
-
+SEXP RTcl_ServiceMode(SEXP args)
+{
+    SEXP ans;
+    int value;
+    
+    if (!isLogical(CADR(args)) || length(CADR(args)) > 1)
+    	error("invalid argument");
+    
+    if (length(CADR(args))) value = Tcl_SetServiceMode(LOGICAL(CADR(args))[0] ? 
+    						TCL_SERVICE_ALL : TCL_SERVICE_NONE);
+    else {
+    	value = Tcl_SetServiceMode(TCL_SERVICE_NONE);
+    	if (value != TCL_SERVICE_NONE) Tcl_SetServiceMode(value); /* Tcl_GetServiceMode was not found */
+    }
+    
+    ans = allocVector(LGLSXP, 1);
+    LOGICAL(ans)[0] = value == TCL_SERVICE_ALL;
+    return ans;
+}
+    
 #ifndef Win32
 #ifndef TCL80
 /* ----- Tcl/Tk console routines ----- */
@@ -844,7 +863,22 @@ RTcl_ReadConsole (char *prompt, unsigned char *buf, int len,
     if (code != TCL_OK)
 	return 0;
     else
+#ifdef SUPPORT_MBCS
+    {
+	    char *buf_utf8;
+	    Tcl_DString buf_utf8_ds;
+	    Tcl_DStringInit(&buf_utf8_ds);
+	    buf_utf8 =
+		    Tcl_UtfToExternalDString(NULL,
+		    			     Tcl_GetStringResult(RTcl_interp),
+					     len,
+					     &buf_utf8_ds);
+            strncpy((char *)buf, buf_utf8, len);
+	    Tcl_DStringFree(&buf_utf8_ds);
+    }
+#else /* SUPPORT_MBCS */
 	strncpy((char *)buf, (char *) Tcl_GetStringResult(RTcl_interp), len);
+#endif /* SUPPORT_MBCS */
 
     /* At some point we need to figure out what to do if the result is
      * longer than "len"... For now, just truncate. */
@@ -861,10 +895,21 @@ static void
 RTcl_WriteConsole (char *buf, int len)
 {
     Tcl_Obj *cmd[2];
+#ifdef SUPPORT_MBCS
+    char *buf_utf8;
+    Tcl_DString  buf_utf8_ds;
+
+    Tcl_DStringInit(&buf_utf8_ds);
+    buf_utf8 = Tcl_ExternalToUtfDString(NULL, buf, -1, &buf_utf8_ds);
+#endif /* SUPPORT_MBCS */
 
     /* Construct command */
     cmd[0] = Tcl_NewStringObj("Rc_write", -1);
+#ifdef SUPPORT_MBCS
+    cmd[1] = Tcl_NewStringObj(buf_utf8, -1);
+#else /* SUPPORT_MBCS */
     cmd[1] = Tcl_NewStringObj(buf, len);
+#endif /* SUPPORT_MBCS */
 
     Tcl_IncrRefCount(cmd[0]);
     Tcl_IncrRefCount(cmd[1]);
@@ -873,6 +918,9 @@ RTcl_WriteConsole (char *buf, int len)
 
     Tcl_DecrRefCount(cmd[0]);
     Tcl_DecrRefCount(cmd[1]);
+#ifdef SUPPORT_MBCS
+    Tcl_DStringFree(&buf_utf8_ds);
+#endif /* SUPPORT_MBCS */
 }
 
 /* Indicate that input is coming from the console */
