@@ -19,20 +19,16 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
-#include "Defn.h" /* remap mainloop */
+/* <UTF8> char here is handled as a whole string */
 
-int Rf_initialize_R(int ac, char **av); /* in system.c */
+int Rf_initialize_R(int ac, char **av); /* in ../unix/system.c */
+void Rf_mainloop();                     /* in main.c */
 
 
 int main(int ac, char **av)
 {
     Rf_initialize_R(ac, av);
-
-    mainloop();
-    /*++++++  in ../main/main.c */
+    Rf_mainloop(); /* does not return */
     return 0;
 }
 
