@@ -22,6 +22,8 @@
 /*  This module contains support for S-style generic */
 /*  functions and "class" support.  Gag, barf ...  */
 
+/* <UTF8> char here is either ASCII or handled as a whole */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -40,8 +42,8 @@ static SEXP GetObject(RCNTXT *cptr)
 	PROTECT(b = findFun(CAR(funcall), sysp));
     else
 	PROTECT(b = eval(CAR(funcall), sysp));
-    /**** use R_sysfunction here insteas */
-    if (TYPEOF(b) != CLOSXP) error("non-closure generic function");
+    /**** use R_sysfunction here instead */
+    if (TYPEOF(b) != CLOSXP) error(_("generic 'function' is not a function"));
     formals = FORMALS(b);
 
     tag = TAG(formals);
@@ -51,7 +53,7 @@ static SEXP GetObject(RCNTXT *cptr)
 	for (b = cptr->promargs ; b != R_NilValue ; b = CDR(b))
 	    if (TAG(b) != R_NilValue && pmatch(tag, TAG(b), 1)) {
 		if ( s != R_NilValue)
-		    error("formal argument \"%s\" matched by multiple actual arguments", tag);
+		    error(_("formal argument \"%s\" matched by multiple actual arguments"), tag);
 		else
 		    s = CAR(b);
 	    }
@@ -61,7 +63,7 @@ static SEXP GetObject(RCNTXT *cptr)
 	    for (b = cptr->promargs ; b != R_NilValue ; b = CDR(b))
 		if (TAG(b) != R_NilValue && pmatch(tag, TAG(b), 0)) {
 		    if ( s != R_NilValue)
-			error("formal argument \"%s\" matched by multiple actual arguments", tag);
+			error(_("formal argument \"%s\" matched by multiple actual arguments"), tag);
 		    else
 			s = CAR(b);
 		}
@@ -172,9 +174,9 @@ SEXP R_LookupMethod(SEXP method, SEXP rho, SEXP callrho, SEXP defrho)
 
     if (R_UseNamespaceDispatch) {
 	if (TYPEOF(callrho) != ENVSXP && callrho != R_NilValue)
-	    error("bad generic call environment");
+	    error(_("bad generic call environment"));
 	if (TYPEOF(defrho) != ENVSXP && defrho != R_NilValue)
-	    error("bad generic definition environment");
+	    error(_("bad generic definition environment"));
 	if (defrho == R_NilValue)
 	    defrho = R_BaseNamespace;
 
@@ -220,7 +222,7 @@ int usemethod(char *generic, SEXP obj, SEXP call, SEXP args,
 
     cptr = R_GlobalContext;
     if ( !(cptr->callflag & CTXT_FUNCTION) || cptr->cloenv != rho)
-	error("UseMethod used in an inappropriate fashion");
+	error(_("'UseMethod' used in an inappropriate fashion"));
 
     /* Create a new environment without any */
     /* of the formals to the generic in it. */
@@ -243,7 +245,7 @@ int usemethod(char *generic, SEXP obj, SEXP call, SEXP args,
 	PROTECT(op);
 	break;
     default:
-	error("Invalid generic function in usemethod");
+	error(_("Invalid generic function in 'usemethod'"));
     }
 
     if (TYPEOF(op) == CLOSXP) {
@@ -261,11 +263,11 @@ int usemethod(char *generic, SEXP obj, SEXP call, SEXP args,
     PROTECT(matchedarg = cptr->promargs);
     PROTECT(newcall = duplicate(cptr->call));
 
-    PROTECT(class = R_data_class(obj, FALSE));
+    PROTECT(class = R_data_class2(obj));
     nclass = length(class);
     for (i = 0; i < nclass; i++) {
 	if(strlen(generic) + strlen(CHAR(STRING_ELT(class, i))) + 2 > 512)
-	    error("class name too long in %s", generic);
+	    error(_("class name too long in '%s'"), generic);
 	sprintf(buf, "%s.%s", generic, CHAR(STRING_ELT(class, i)));
 	method = install(buf);
 	sxp = R_LookupMethod(method, rho, callrho, defrho);
@@ -304,7 +306,7 @@ int usemethod(char *generic, SEXP obj, SEXP call, SEXP args,
 	}
     }
     if(strlen(generic) + strlen("default") + 2 > 512)
-	error("class name too long in %s", generic);
+	error(_("class name too long in '%s'"), generic);
     sprintf(buf, "%s.default", generic);
     method = install(buf);
     sxp = R_LookupMethod(method, rho, callrho, defrho);
@@ -339,7 +341,7 @@ int usemethod(char *generic, SEXP obj, SEXP call, SEXP args,
 
 SEXP do_usemethod(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP ans, generic, obj;
+    SEXP ans, generic = R_NilValue /* -Wall */, obj;
     SEXP callenv, defenv;
     int nargs;
     RCNTXT *cptr;
@@ -347,24 +349,24 @@ SEXP do_usemethod(SEXP call, SEXP op, SEXP args, SEXP env)
     nargs = length(args);
 
     if (nargs < 0)
-	errorcall(call, "corrupt internals!");
+	errorcall(call, _("corrupt internals!"));
 
     /* get environments needed for dispatching.
        callenv = environment from which the generic was called
        defenv = environment where the generic was defined */
     cptr = R_GlobalContext;
     if ( !(cptr->callflag & CTXT_FUNCTION) || cptr->cloenv != env)
-	error("UseMethod used in an inappropriate fashion");
+	error(_("'UseMethod' used in an inappropriate fashion"));
     callenv = cptr->sysparent;
     defenv = TYPEOF(env) == ENVSXP ? ENCLOS(env) : R_NilValue;
 
     if (nargs)
 	PROTECT(generic = eval(CAR(args), env));
-    else
-	generic = R_MissingArg;
+    else 
+	errorcall(call, _("there must be a first argument"));
 
     if (nargs > 2)  /* R-lang says there should be a warning */
-	warningcall(call, "Arguments after the first two are ignored");
+	warningcall(call, _("arguments after the first two are ignored"));
     if (nargs >= 2)
 	PROTECT(obj = eval(CADR(args), env));
     else {
@@ -375,16 +377,16 @@ SEXP do_usemethod(SEXP call, SEXP op, SEXP args, SEXP env)
 	    cptr = cptr->nextcontext;
 	}
 	if (cptr == NULL)
-	    error("UseMethod called from outside a closure");
-	if (generic == R_MissingArg)
-	    PROTECT(generic = mkString(CHAR(PRINTNAME(CAR(cptr->call)))));
+	    error(_("'UseMethod' called from outside a closure"));
+	/* if (generic == R_MissingArg)
+	   PROTECT(generic = mkString(CHAR(PRINTNAME(CAR(cptr->call))))); */
 	PROTECT(obj = GetObject(cptr));
     }
 
     if (TYPEOF(generic) != STRSXP ||
 	LENGTH(generic) < 1 ||
 	strlen(CHAR(STRING_ELT(generic, 0))) == 0)
-	errorcall(call, "first argument must be a generic name");
+	errorcall(call, _("first argument must be a generic name"));
 
     if (usemethod(CHAR(STRING_ELT(generic, 0)), obj, call, CDR(args),
 		  env, callenv, defenv, &ans) == 1) {
@@ -394,7 +396,8 @@ SEXP do_usemethod(SEXP call, SEXP op, SEXP args, SEXP env)
 	UNPROTECT(1);
     }
     else
-	error("no applicable method for \"%s\"", CHAR(STRING_ELT(generic, 0)));
+	error(_("no applicable method for \"%s\""), 
+	      CHAR(STRING_ELT(generic, 0)));
     return R_NilValue; /* NOT Used */
 }
 
@@ -453,13 +456,13 @@ SEXP do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
 	cptr = cptr->nextcontext;
     }
     if (cptr == NULL)
-	error("NextMethod called from outside a closure");
+	error(_("'NextMethod' called from outside a function"));
 
     PROTECT(newcall = duplicate(cptr->call));
 
     /* eg get("print.ts")(1) */
     if (TYPEOF(CAR(cptr->call)) == LANGSXP)
-       error("NextMethod called from anonymous function");
+       error(_("'NextMethod' called from an anonymous function"));
 
     /* Find dispatching environments. Promises shouldn't occur, but
        check to be on the safe side.  If the variables are not in the
@@ -489,9 +492,9 @@ SEXP do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
     if (TYPEOF(s) == PROMSXP)  /* looks like R_LookupMethod just did this */
 	s = eval(s, env);
     if (TYPEOF(s) == SYMSXP && s == R_UnboundValue) 
-	error("No calling generic was found: was a method called directly?");
+	error(_("no calling generic was found: was a method called directly?"));
     if (TYPEOF(s) != CLOSXP){
-	errorcall(R_NilValue, "function is not a closure, but of type %d", 
+	errorcall(R_NilValue, _("'function' is not a function, but of type %d"), 
 		  TYPEOF(s));
     }
     /* get formals and actuals; attach the names of the formals to
@@ -567,7 +570,7 @@ SEXP do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
 	}
     }
     else
-	errorcall(call,"wrong argument ...");
+	errorcall(call, _("wrong argument ..."));
 
     /*
       .Class is used to determine the next method; if it doesn't
@@ -581,7 +584,7 @@ SEXP do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
     if (class == R_UnboundValue) {
 	s = GetObject(cptr);
 	if (!isObject(s))
-	    errorcall(call, "object not specified");
+	    errorcall(call, _("object not specified"));
 	class = getAttrib(s, R_ClassSymbol);
     }
 
@@ -591,14 +594,14 @@ SEXP do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
     if (generic == R_UnboundValue)
 	generic = eval(CAR(args), env);
     if( generic == R_NilValue )
-	errorcall(call,"generic function not specified");
+	errorcall(call, _("generic function not specified"));
     PROTECT(generic);
 
     if (!isString(generic) || length(generic) > 1)
-	errorcall(call,"invalid generic argument to NextMethod");
+	errorcall(call, _("invalid generic argument to NextMethod"));
 
     if (strlen(CHAR(STRING_ELT(generic, 0))) == 0)
-	errorcall(call,"generic function not specified");
+	errorcall(call, _("generic function not specified"));
 
     /* determine whether we are in a Group dispatch */
 
@@ -610,7 +613,7 @@ SEXP do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
 	PROTECT(group);
 
     if (!isString(group) || length(group) > 1)
-	errorcall(call, "invalid group argument found in NextMethod");
+	errorcall(call, _("invalid 'group' argument found in NextMethod"));
 
     /* determine the root: either the group or the generic will be it */
 
@@ -627,10 +630,11 @@ SEXP do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
     method = findVarInFrame3(R_GlobalContext->sysparent,install(".Method"), TRUE);
     if( method != R_UnboundValue) {
 	if( !isString(method) )
-	    error("Wrong value for .Method");
+	    error(_("wrong value for .Method"));
 	for( i = 0; i < length(method); i++ ) {
 	if(strlen(CHAR(STRING_ELT(method, i))) >= 512)
-	    error("method name too long in %s", CHAR(STRING_ELT(method, i)));
+	    error(_("method name too long in '%s'"),
+		  CHAR(STRING_ELT(method, i)));
 	  sprintf(b, "%s", CHAR(STRING_ELT(method, i)));
 	  if( strlen(b) )
 	    break;
@@ -639,22 +643,25 @@ SEXP do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
 	   is the same or absent */
 	for(j = i; j < length(method); j++){
 	if(strlen(CHAR(STRING_ELT(method, j))) >= 512)
-	    error("method name too long in %s", CHAR(STRING_ELT(method, j)));
+	    error(_("method name too long in '%s'"),
+		  CHAR(STRING_ELT(method, j)));
 	  sprintf(bb, "%s",CHAR(STRING_ELT(method, j)));
 	  if (strlen(bb) && strcmp(b,bb))
-	      warning("Incompatible methods ignored");
+	      warning(_("Incompatible methods ignored"));
 	}
     }
     else {
 	if(strlen(CHAR(PRINTNAME(CAR(cptr->call)))) >= 512)
-	   error("call name too long in %s", CHAR(PRINTNAME(CAR(cptr->call))));
+	   error(_("call name too long in '%s'"),
+		 CHAR(PRINTNAME(CAR(cptr->call))));
       sprintf(b, "%s", CHAR(PRINTNAME(CAR(cptr->call))));
     }
 
     for (j = 0; j < length(class); j++) {
 	if(strlen(CHAR(STRING_ELT(basename, 0))) + 
 	   strlen(CHAR(STRING_ELT(class, j))) + 2 > 512)
-	    error("class name too long in %s", CHAR(STRING_ELT(basename, 0)));
+	    error(_("class name too long in '%s'"),
+		  CHAR(STRING_ELT(basename, 0)));
 	sprintf(buf, "%s.%s", CHAR(STRING_ELT(basename, 0)),
 		CHAR(STRING_ELT(class, j)));
 	if ( !strcmp(buf, b) )
@@ -671,7 +678,8 @@ SEXP do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
     for (i = j ; i < length(class); i++) {
 	if(strlen(CHAR(STRING_ELT(generic, 0))) + 
 	   strlen(CHAR(STRING_ELT(class, i))) + 2 > 512)
-	    error("class name too long in %s", CHAR(STRING_ELT(generic, 0)));
+	    error(_("class name too long in '%s'"),
+		  CHAR(STRING_ELT(generic, 0)));
 	sprintf(buf, "%s.%s", CHAR(STRING_ELT(generic, 0)),
 		CHAR(STRING_ELT(class, i)));
 	nextfun = R_LookupMethod(install(buf), env, callenv, defenv);
@@ -703,12 +711,12 @@ SEXP do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
 	    if (TYPEOF(nextfun)==PROMSXP)
 		nextfun = eval(nextfun, env);
 	    if (!isFunction(nextfun))
-		error("No method to invoke");
+		error(_("no method to invoke"));
 	    if (TYPEOF(nextfun) == CLOSXP) {
 		if (INTERNAL(t) != R_NilValue)
 		    nextfun = INTERNAL(t);
 		else
-		    error("No method to invoke");
+		    error(_("no method to invoke"));
 	    }
 	}
     }
@@ -745,6 +753,16 @@ SEXP do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
 SEXP do_unclass(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
+    switch(TYPEOF(CAR(args))) {
+    case ENVSXP:
+	errorcall(call, _("cannot unclass an environment"));
+	break;
+    case EXTPTRSXP:
+	errorcall(call, _("cannot unclass an external pointer"));
+	break;
+    default:
+	break;
+    }
     if (isObject(CAR(args))) {
 	SETCAR(args, duplicate(CAR(args)));
 	setAttrib(CAR(args), R_ClassSymbol, R_NilValue);
@@ -816,12 +834,12 @@ SEXP do_inherits(SEXP call, SEXP op, SEXP args, SEXP env)
 
     what = CADR(args);
     if( !isString(what) )
-	errorcall(call, "what must be a character vector");
+	errorcall(call, _("'what' must be a character vector"));
     nwhat = length(what);
 
     which = CADDR(args);
     if( !isLogical(which) || (length(which) != 1) )
-	errorcall(call, "which must be a length 1 logical vector");
+	errorcall(call, _("'which' must be a length 1 logical vector"));
     isvec = asLogical(which);
 
     if( isvec )
@@ -916,17 +934,19 @@ static SEXP dispatchNonGeneric(SEXP name, SEXP env, SEXP fdef)
 	fun = findVarInFrame3(rho, symbol, TRUE);
 	if(fun == R_UnboundValue) continue;
 	switch(TYPEOF(fun)) {
-	case BUILTINSXP:  case SPECIALSXP: break;
 	case CLOSXP:
 	    value = findVarInFrame3(CLOENV(fun), dot_Generic, TRUE);
 	    if(value == R_UnboundValue) break;
-	    /*in all other cases, go on to the parent environment */
+	case BUILTINSXP:  case SPECIALSXP:
+	default:
+	    /* in all other cases, go on to the parent environment */
+	    break;
 	}
 	fun = R_UnboundValue;
     }
     fun = SYMVALUE(symbol);
     if(fun == R_UnboundValue)
-	error("Unable to find a non-generic version of function \"%s\"",
+	error(_("unable to find a non-generic version of function \"%s\""),
 	      CHAR(asChar(name)));
     cptr = R_GlobalContext;
     /* check this is the right context */
@@ -966,19 +986,19 @@ SEXP do_standardGeneric(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP arg, value, fdef; R_stdGen_ptr_t ptr = R_get_standardGeneric_ptr();
     if(!ptr) {
-	warning("standardGeneric called without methods dispatch enabled (will be ignored)");
+	warning(_("standardGeneric called without methods dispatch enabled (will be ignored)"));
 	R_set_standardGeneric_ptr(dispatchNonGeneric, NULL);
 	ptr = R_get_standardGeneric_ptr();
     }
     PROTECT(args);
     PROTECT(arg = CAR(args));
     if(!isValidStringF(arg))
-      error("Argument to standardGeneric must be a non-empty character string");
+      error(_("argument to standardGeneric must be a non-empty character string"));
 
     PROTECT(fdef = get_this_generic(args));
 
     if(isNull(fdef))
-      error("Call to standardGeneric(\"%s\") apparently not from the body of that generic function", CHAR(STRING_ELT(arg, 0)));
+      error(_("call to standardGeneric(\"%s\") apparently not from the body of that generic function"), CHAR(STRING_ELT(arg, 0)));
 
     value = (*ptr)(arg, env, fdef);
 
@@ -999,7 +1019,7 @@ SEXP R_set_prim_method(SEXP fname, SEXP op, SEXP code_vec, SEXP fundef,
 {
     char *code_string;
     if(!isValidString(code_vec))
-	error("Argument \"code\" must be a character string");
+	error(_("argument 'code' must be a character string"));
     code_string = CHAR(asChar(code_vec));
     do_set_prim_method(op, code_string, fundef, mlist);
     return(fname);
@@ -1038,7 +1058,7 @@ SEXP do_set_prim_method(SEXP op, char *code_string, SEXP fundef, SEXP mlist)
 	errorcase = TRUE;
     }
     if(errorcase) {
-	error("Invalid primitive methods code (\"%s\"): should be \"clear\", \"reset\", \"set\", or \"suppress\"", code_string);
+	error(_("invalid primitive methods code (\"%s\"): should be \"clear\", \"reset\", \"set\", or \"suppress\""), code_string);
 	return R_NilValue;
     }
     switch(TYPEOF(op)) {
@@ -1046,7 +1066,7 @@ SEXP do_set_prim_method(SEXP op, char *code_string, SEXP fundef, SEXP mlist)
 	offset = PRIMOFFSET(op);
 	break;
     default:
-	error("Invalid object: must be a primitive function");
+	error(_("invalid object: must be a primitive function"));
     }
     if(offset >= maxMethodsOffset) {
 	int n;
@@ -1093,7 +1113,7 @@ SEXP do_set_prim_method(SEXP op, char *code_string, SEXP fundef, SEXP mlist)
     }
     else if(fundef && !isNull(fundef) && !prim_generics[offset]) {
 	if(TYPEOF(fundef) != CLOSXP)
-	    error("The formal definition of a primitive generic must be a function object (got type %s)",
+	    error(_("the formal definition of a primitive generic must be a function object (got type '%s')"),
 		  type2str(TYPEOF(fundef)));
 	R_PreserveObject(fundef);
 	prim_generics[offset] = fundef;
@@ -1207,7 +1227,7 @@ SEXP R_possible_dispatch(SEXP call, SEXP op, SEXP args, SEXP rho)
     prim_methods_t current;
     offset = PRIMOFFSET(op);
     if(offset < 0 || offset > curMaxOffset)
-	error("Invalid primitive operation given for dispatch");
+	error(_("invalid primitive operation given for dispatch"));
     current = prim_methods[offset];
     if(current == NO_METHODS || current == SUPPRESSED)
 	return(NULL);
@@ -1236,7 +1256,7 @@ SEXP R_possible_dispatch(SEXP call, SEXP op, SEXP args, SEXP rho)
     }
     fundef = prim_generics[offset];
     if(!fundef || TYPEOF(fundef) != CLOSXP)
-	error("primitive function \"%s\" has been set for methods but no  generic function supplied",
+	error(_("primitive function \"%s\" has been set for methods but no generic function supplied"),
 	      PRIMNAME(op));
     /* To do:  arrange for the setting to be restored in case of an
        error in method search */
@@ -1253,7 +1273,7 @@ SEXP R_do_MAKE_CLASS( char *what)
     static SEXP s_getClass = NULL;
     SEXP e, call;
     if(!what)
-	error("C level MAKE_CLASS macro called with NULL string pointer");
+	error(_("C level MAKE_CLASS macro called with NULL string pointer"));
     if(!s_getClass)
 	s_getClass = Rf_install("getClass");
     PROTECT(call = allocVector(LANGSXP, 2));
@@ -1274,11 +1294,11 @@ SEXP R_do_new_object(SEXP class_def)
 	s_className = Rf_install("className");
     }
     if(!class_def)
-	error("C level NEW macro called with null class definition pointer");
+	error(_("C level NEW macro called with null class definition pointer"));
     e = R_do_slot(class_def, s_virtual);
     if(asLogical(e) != 0)  { /* includes NA, TRUE, or anything other than FALSE */
 	e = R_do_slot(class_def, s_className);
-	error("Trying to generate an object in C from a virtual class (\"%s\")",
+	error(_("trying to generate an object in C from a virtual class (\"%s\")"),
 	      CHAR(asChar(e)));
     }
     e = R_do_slot(class_def, s_className);
