@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2000   The R Development Core Team.
+ *  Copyright (C) 1998-2005   The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -273,10 +273,6 @@ void InitOptions(void)
     LOGICAL(CAR(v))[0] = R_KeepSource;
     v = CDR(v);
 
-    SET_TAG(v, install("error.messages"));
-    SETCAR(v, allocVector(LGLSXP, 1));
-    LOGICAL(CAR(v))[0] = 1;
-
     SET_TAG(v, install("warnings.length"));
     SETCAR(v, allocVector(INTSXP, 1));
     INTEGER(CAR(v))[0] = 1000;
