@@ -17,6 +17,10 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/* <UTF8> the only interpretation of char is ASCII 
+   <MBCS> all the char quantities should be ASCII
+ */
+
 
 /* based on libxml2-2.3.6:
  * nanohttp.c: minimalist HTTP GET implementation to fetch external subsets.
@@ -36,6 +40,18 @@
 #include <config.h>
 #endif
 
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#ifdef Win32
+#define _(String) libintl_gettext (String)
+#undef gettext /* needed for graphapp */
+#else
+#define _(String) gettext (String)
+#endif
+#else /* not NLS */
+#define _(String) (String)
+#endif
+
 #if !defined(Unix) || defined(HAVE_BSD_NETWORKING)
 
 #ifdef Win32
@@ -388,9 +404,9 @@ RxmlNanoHTTPScanProxy(const char *URL)
 	proxyPort = 0;
 	}*/
     if (URL == NULL)
-	RxmlMessage(0, "Removing HTTP proxy info");
+	RxmlMessage(0, _("removing HTTP proxy info"));
     else
-	RxmlMessage(1, "Using HTTP proxy %s", URL);
+	RxmlMessage(1, _("using HTTP proxy '%s'"), URL);
     if (URL == NULL) return;
     buf[indx] = 0;
     while (*cur != 0) {
@@ -961,7 +977,7 @@ RxmlNanoHTTPConnectHost(const char *host, int port)
     h=gethostbyname(host);
     if (h==NULL)
     {
-	RxmlMessage(2, "unable to resolve '%s'.", host);
+	RxmlMessage(2, _("unable to resolve '%s'."), host);
 	return(-1);
     }
 
@@ -988,12 +1004,12 @@ RxmlNanoHTTPConnectHost(const char *host, int port)
 
 	s = RxmlNanoHTTPConnectAttempt(addr);
 	if (s != -1) {
-	    RxmlMessage(1, "connected to '%s' on port %d.", host, port);
+	    RxmlMessage(1, _("connected to '%s' on port %d."), host, port);
 	    return(s);
 	}
     }
 
-    RxmlMessage(2, "unable to connect to '%s' on port %d.", host, port);
+    RxmlMessage(2, _("unable to connect to '%s' on port %d."), host, port);
     return(-1);
 }
 
@@ -1257,7 +1273,7 @@ RxmlNanoHTTPMethod(const char *URL, const char *method, const char *input,
 
     if ((ctxt->location != NULL) && (ctxt->returnValue >= 300) &&
         (ctxt->returnValue < 400)) {
-	RxmlMessage(1, "Redirect to: %s", ctxt->location);
+	RxmlMessage(1, _("redirect to: '%s'"), ctxt->location);
 	while (RxmlNanoHTTPRecv(ctxt)) ;
         if (nbRedirects < XML_NANO_HTTP_MAX_REDIR) {
 	    nbRedirects++;
@@ -1266,7 +1282,7 @@ RxmlNanoHTTPMethod(const char *URL, const char *method, const char *input,
 	    goto retry;
 	}
 	RxmlNanoHTTPFreeCtxt(ctxt);
-	RxmlMessage(2, "Too many redirects, aborting ...");
+	RxmlMessage(2, _("too many redirects, aborting ..."));
 	return(NULL);
 
     }
