@@ -36,16 +36,26 @@ double pf(double x, double n1, double n2, int lower_tail, int log_p)
     if (x <= 0.)
 	return R_DT_0;
 
-    /* fudge the extreme DF cases -- pbeta doesn't do this well
+    /* move to pchisq for very large values - was 'n1 > 4e5' in 2.0.x,
+       now only needed for n1 = Inf or n2 = Inf {since pbeta(0,*)=0} : */
+    if (n2 == ML_POSINF) {
+	if (n1 == ML_POSINF) {
+	    if(x <  1.) return R_DT_0;
+	    if(x == 1.) return (log_p ? -M_LN2 : 0.5);
+	    if(x >  1.) return R_DT_1;
+	}
 
-    if (n2 > 4e5)
 	return pchisq(x * n1, n1, lower_tail, log_p);
+    }
 
-    if (n1 > 4e5)
-	return pchisq(n2 / x , n2, !lower_tail, log_p); 
-    */
+    if (n1 == ML_POSINF)/* was "fudge"	'n1 > 4e5' in 2.0.x */
+	return pchisq(n2 / x , n2, !lower_tail, log_p);
 
-    x = pbeta(n2 / (n2 + n1 * x), n2 / 2.0, n1 / 2.0, !lower_tail, log_p);
+    /* Avoid squeezing pbeta's first parameter against 1 :  */
+    if (n1 * x > n2)
+	x = pbeta(n2 / (n2 + n1 * x),	  n2 / 2., n1 / 2., !lower_tail, log_p);
+    else
+	x = pbeta(n1 * x / (n2 + n1 * x), n1 / 2., n2 / 2.,  lower_tail, log_p);
 
     return ML_VALID(x) ? x : ML_NAN;
 }
