@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2003  Robert Gentleman, Ross Ihaka
+ *  Copyright (C) 1997--2004  Robert Gentleman, Ross Ihaka
  *                            and the R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,14 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/* <UTF8> char here is mainly handled as a whole string.
+   Does need readline to support it.
+   Appending \n\0 is OK in UTF-8, not general MBCS.
+   Removal of \r is OK on UTF-8.
+   ? use of isspace OK?
+ */
+
+
 /* See system.txt for a description of functions */
 
 #ifdef HAVE_CONFIG_H
@@ -35,20 +43,7 @@
 #include "Rdevices.h"		/* for KillAllDevices */
 #include "Runix.h"
 #include "Startup.h"
-
-#ifdef HAVE_LIBREADLINE
-# ifdef HAVE_READLINE_READLINE_H
-#  include <readline/readline.h>
-# endif
-# ifdef HAVE_READLINE_HISTORY_H
-#  include <readline/history.h>
-# endif
-#endif
-
-/* For compatibility with pre-readline4.2 systems: */
-#if !defined (_RL_FUNCTION_TYPEDEF)
-typedef void rl_vcpfunc_t (char *);
-#endif /* _RL_FUNCTION_TYPEDEF */
+#include <R_ext/Riconv.h>
 
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>		/* for unlink */
@@ -67,7 +62,8 @@ extern Rboolean UsingReadline;
 
 void Rstd_Suicide(char *s)
 {
-    REprintf("Fatal error: %s\n", s);
+    REprintf("Fatal error: %s\n", s); 
+    /* Might be called before translation is running */
     R_CleanUp(SA_SUICIDE, 2, 0);
 }
 
@@ -121,7 +117,7 @@ int R_SelectEx(int  n,  fd_set  *readfds,  fd_set  *writefds,
 	volatile sel_intr_handler_t myintr = intr != NULL ? intr : onintr;
 	if (SIGSETJMP(seljmpbuf, 1)) {
 	    myintr();
-	    error("interrupt handler must not return");
+	    error(_("interrupt handler must not return"));
 	    return 0; /* not reached */
 	}
 	else {
@@ -390,6 +386,37 @@ getSelectedHandler(InputHandler *handlers, fd_set *readMask)
 
 
 #ifdef HAVE_LIBREADLINE
+
+# ifdef HAVE_READLINE_READLINE_H
+#  include <readline/readline.h>
+/* For compatibility with pre-readline4.2 systems: */
+#  if !defined (_RL_FUNCTION_TYPEDEF)
+typedef void rl_vcpfunc_t (char *);
+#  endif /* _RL_FUNCTION_TYPEDEF */
+# else
+typedef void rl_vcpfunc_t (char *);
+extern void rl_callback_handler_install(const char *, rl_vcpfunc_t *);
+extern void rl_callback_handler_remove(void);
+extern void rl_callback_read_char(void);
+extern char *tilde_expand (const char *);
+# endif
+
+char *R_ExpandFileName_readline(char *s, char *buff)
+{
+    char *s2 = tilde_expand(s);
+
+    strncpy(buff, s2, PATH_MAX);
+    if(strlen(s2) >= PATH_MAX) buff[PATH_MAX-1] = '\0';
+    free(s2);
+    return buff;
+}
+
+
+# ifdef HAVE_READLINE_HISTORY_H
+#  include <readline/history.h>
+# endif
+
+
 /* callback for rl_callback_read_char */
 
 
@@ -453,11 +480,14 @@ void
 pushReadline(char *prompt, rl_vcpfunc_t f)
 {
    if(ReadlineStack.current >= ReadlineStack.max) {
-     warning("An unusual circumstance has arisen in the nesting of readline input. Please report using bug.report()");
+     warning(_("An unusual circumstance has arisen in the nesting of readline input. Please report using bug.report()"));
    } else
      ReadlineStack.fun[++ReadlineStack.current] = f;
 
    rl_callback_handler_install(prompt, f);
+   /* flush stdout in case readline wrote the prompt, but didn't flush
+      stdout to make it visible. (needed for Apple's rl in OS X 10.4-pre) */
+   fflush(stdout);
 }
 
 /*
@@ -533,12 +563,13 @@ handleInterrupt(void)
 #endif /* HAVE_LIBREADLINE */
 
 /* Fill a text buffer from stdin or with user typed console input. */
+static void *cd = NULL;
 
 int Rstd_ReadConsole(char *prompt, unsigned char *buf, int len,
 		     int addtohistory)
 {
     if(!R_Interactive) {
-	int ll;
+	int ll, err = 0;
 	if (!R_Slave)
 	    fputs(prompt, stdout);
 	if (fgets((char *)buf, len, stdin) == NULL)
@@ -549,9 +580,33 @@ int Rstd_ReadConsole(char *prompt, unsigned char *buf, int len,
 	    buf[ll - 2] = '\n';
 	    buf[--ll] = '\0';
 	}
+	/* translate if necessary */
+	if(strlen(R_StdinEnc) && strcmp(R_StdinEnc, "native.enc")) {
+#ifdef HAVE_DECL_ICONV
+	    size_t res, inb = strlen((char *)buf), onb = len;
+	    char obuf[1001];
+	    char *ib = (char *)buf, *ob = obuf;
+	    if(!cd) {
+		cd = Riconv_open("", R_StdinEnc);
+		if(!cd) error(_("encoding '%s' is not recognised"), R_StdinEnc);
+	    }
+	    res = Riconv(cd, &ib, &inb, &ob, &onb);
+	    *ob = '\0';
+	    err = res == (size_t)(-1);
+	    /* errors lead to part of the input line being ignored */
+	    if(err) fputs(_("<ERROR: invalid input in encoding> "), stdout);
+	    strncpy((char *)buf, obuf, len);
+#else
+	    if(!cd) {
+		warning(_("re-encoding is not available on this system"));
+		cd = (void *)1;
+	    }
+#endif
+    	}
 /* according to system.txt, should be terminated in \n, so check this
-   at eof */
-	if (feof(stdin) && (ll == 0 || buf[ll - 1] != '\n') && ll < len) {
+   at eof and error */
+	if ((err || feof(stdin)) 
+	    && (ll == 0 || buf[ll - 1] != '\n') && ll < len) {
 	    buf[ll++] = '\n'; buf[ll] = '\0';
 	}
 	if (!R_Slave)
@@ -713,6 +768,7 @@ void Rstd_CleanUp(SA_TYPE saveact, int status, int runLast)
 #ifdef HAVE_LIBREADLINE
 # ifdef HAVE_READLINE_HISTORY_H
 	if(R_Interactive && UsingReadline) {
+	    R_setupHistory(); /* re-read the history size and filename */
 	    stifle_history(R_HistorySize);
 	    write_history(R_HistoryFile);
 	}
@@ -840,21 +896,20 @@ void Rstd_loadhistory(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP sfile;
     char file[PATH_MAX], *p;
 
-    checkArity(op, args);
     sfile = CAR(args);
     if (!isString(sfile) || LENGTH(sfile) < 1)
-	errorcall(call, "invalid file argument");
+	errorcall(call, _("invalid 'file' argument"));
     p = R_ExpandFileName(CHAR(STRING_ELT(sfile, 0)));
     if(strlen(p) > PATH_MAX - 1)
-	errorcall(call, "file argument is too long");
+	errorcall(call, _("'file' argument is too long"));
     strcpy(file, p);
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_READLINE_HISTORY_H)
     if(R_Interactive && UsingReadline) {
 	clear_history();
 	read_history(file);
-    } else errorcall(call, "no history mechanism available");
+    } else errorcall(call, _("no history mechanism available"));
 #else
-    errorcall(call, "no history mechanism available");
+    errorcall(call, _("no history mechanism available"));
 #endif
 }
 
@@ -863,21 +918,23 @@ void Rstd_savehistory(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP sfile;
     char file[PATH_MAX], *p;
 
-    checkArity(op, args);
     sfile = CAR(args);
     if (!isString(sfile) || LENGTH(sfile) < 1)
-	errorcall(call, "invalid file argument");
+	errorcall(call, _("invalid 'file' argument"));
     p = R_ExpandFileName(CHAR(STRING_ELT(sfile, 0)));
     if(strlen(p) > PATH_MAX - 1)
-	errorcall(call, "file argument is too long");
+	errorcall(call, _("'file' argument is too long"));
     strcpy(file, p);
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_READLINE_HISTORY_H)
     if(R_Interactive && UsingReadline) {
 	write_history(file);
+#ifdef HAVE_HISTORY_TRUNCATE_FILE
+	R_setupHistory(); /* re-read the history size */
 	history_truncate_file(file, R_HistorySize);
-    } else errorcall(call, "no history available to save");
+#endif
+    } else errorcall(call, _("no history available to save"));
 #else
-    errorcall(call, "no history available to save");
+    errorcall(call, _("no history available to save"));
 #endif
 }
 
@@ -913,7 +970,7 @@ SEXP do_syssleep(SEXP call, SEXP op, SEXP args, SEXP rho)
     checkArity(op, args);
     timeint = asReal(CAR(args));
     if (ISNAN(timeint) || timeint < 0)
-	errorcall(call, "invalid time value");
+	errorcall(call, _("invalid 'time' value"));
     tm = timeint * 1e6;
 
     start = times(&timeinfo);
@@ -942,7 +999,7 @@ SEXP do_syssleep(SEXP call, SEXP op, SEXP args, SEXP rho)
 #else /* not _R_HAVE_TIMING_ */
 SEXP do_syssleep(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    error("Sys.sleep is not implemented on this system");
+    error(_("Sys.sleep is not implemented on this system"));
     return R_NilValue;		/* -Wall */
 }
 #endif /* not _R_HAVE_TIMING_ */
